use anyhow::Result;
use crawl::wgpu;

pub struct VoxelRenderer {
    render_texture: crawl::Texture,
    render_pipeline: wgpu::RenderPipeline,
}

impl VoxelRenderer {
    pub fn new(context: &crawl::Context) -> Result<Self> {
        let shader = context
            .device
            .create_shader_module(wgpu::include_wgsl!(concat!(
                env!("CARGO_MANIFEST_DIR"),
                "/res/quad.wgsl"
            )));

        let render_texture = crawl::TextureBuilder::new()
            .with_size(context.size.width, context.size.height, 1)
            .with_format(wgpu::TextureFormat::Rgba8Unorm)
            .with_usage(
                wgpu::TextureUsages::TEXTURE_BINDING
                    | wgpu::TextureUsages::COPY_DST
                    | wgpu::TextureUsages::STORAGE_BINDING,
            )
            .with_shader_visibility(wgpu::ShaderStages::FRAGMENT | wgpu::ShaderStages::COMPUTE)
            .build(context)?;

        let render_pipeline = crawl::RenderPipelineBuilder::new("Raycast Quad", &shader)
            .with_layout("Draw", &[&render_texture.bind_group_layout], &[])
            .with_fragment_targets(&[Some(context.surface_config.format.into())])
            .build(context);

        Ok(Self {
            render_texture,
            render_pipeline,
        })
    }
}

impl crawl::Pass for VoxelRenderer {
    fn execute(&self, encoder: &mut wgpu::CommandEncoder, view: &wgpu::TextureView) {
        let mut render_pass = encoder.begin_render_pass(&wgpu::RenderPassDescriptor {
            label: Some("Render Pass"),
            color_attachments: &[Some(wgpu::RenderPassColorAttachment {
                view,
                resolve_target: None,
                ops: wgpu::Operations::default(),
            })],
            ..Default::default()
        });
        render_pass.set_pipeline(&self.render_pipeline);
        render_pass.set_bind_group(0, &self.render_texture.bind_group, &[]);
        render_pass.draw(0..6, 0..1);
    }
}
